/// <reference types="codemirror" />
/// <reference types="react" />
declare module "SimpleMdeReact" {
    import React from "react";
    import SimpleMDE from "easymde";
    import type { Doc, Editor, EditorChange, EditorChangeCancellable, EditorChangeLinkedList, EditorSelectionChange, KeyMap, LineHandle, Position } from "codemirror";
    export type DOMEvent = "mousedown" | "dblclick" | "touchstart" | "contextmenu" | "keydown" | "keypress" | "keyup" | "cut" | "copy" | "paste" | "dragstart" | "dragenter" | "dragover" | "dragleave" | "drop";
    export type CopyEvents = {
        [TKey in string & DOMEvent & keyof DocumentAndElementEventHandlersEventMap as `${TKey}`]?: (instance: Editor, event: DocumentAndElementEventHandlersEventMap[TKey]) => void;
    };
    export type GlobalEvents = {
        [TKey in string & DOMEvent & keyof GlobalEventHandlersEventMap as `${TKey}`]?: (instance: Editor, event: GlobalEventHandlersEventMap[TKey]) => void;
    };
    export type DefaultEvent = (instance: Editor, ...args: any[]) => void;
    export type IndexEventsSignature = {
        [key: string]: DefaultEvent | undefined;
    };
    export interface SimpleMdeToCodemirrorEvents extends CopyEvents, GlobalEvents, IndexEventsSignature {
        change?: (instance: Editor, changeObj: EditorChangeLinkedList) => void;
        changes?: (instance: Editor, changes: EditorChangeLinkedList[]) => void;
        beforeChange?: (instance: Editor, changeObj: EditorChangeCancellable) => void;
        cursorActivity?: (instance: Editor) => void;
        keyHandled?: (instance: Editor, name: string, event: KeyboardEvent) => void;
        inputRead?: (instance: Editor, changeObj: EditorChange) => void;
        electricInput?: (instance: Editor, line: number) => void;
        beforeSelectionChange?: (instance: Editor, obj: EditorSelectionChange) => void;
        viewportChange?: (instance: Editor, from: number, to: number) => void;
        swapDoc?: (instance: Editor, oldDoc: Doc) => void;
        gutterClick?: (instance: Editor, line: number, gutter: string, clickEvent: MouseEvent) => void;
        gutterContextMenu?: (instance: Editor, line: number, gutter: string, contextMenu: MouseEvent) => void;
        /** Fires whenever the editor is focused. */
        focus?: (instance: Editor, event: FocusEvent) => void;
        /** Fires whenever the editor is unfocused. */
        blur?: (instance: Editor, event: FocusEvent) => void;
        /** Fires when the editor is scrolled. */
        scroll?: (instance: Editor) => void;
        /** Fires when the editor is refreshed or resized. Mostly useful to invalidate cached values that depend on the editor or character size. */
        refresh?: (instance: Editor) => void;
        /** Dispatched every time an option is changed with setOption. */
        optionChange?: (instance: Editor, option: string) => void;
        /** Fires when the editor tries to scroll its cursor into view. Can be hooked into to take care of additional scrollable containers around the editor. When the event object has its preventDefault method called, CodeMirror will not itself try to scroll the window. */
        scrollCursorIntoView?: (instance: Editor, event: Event) => void;
        /** Will be fired whenever CodeMirror updates its DOM display. */
        update?: (instance: Editor) => void;
        /** Fired whenever a line is (re-)rendered to the DOM. Fired right after the DOM element is built, before it is added to the document.
         The handler may mess with the style of the resulting element, or add event handlers, but should not try to change the state of the editor. */
        renderLine?: (instance: Editor, line: LineHandle, element: HTMLElement) => void;
        /** Fires when the overwrite flag is flipped. */
        overwriteToggle?: (instance: Editor, overwrite: boolean) => void;
    }
    export type GetMdeInstance = (instance: SimpleMDE) => void;
    export type GetCodemirrorInstance = (instance: Editor) => void;
    export type GetLineAndCursor = (instance: Position) => void;
    export interface SimpleMDEReactProps extends Omit<React.HTMLAttributes<HTMLDivElement>, "onChange"> {
        id?: string;
        onChange?: (value: string) => void;
        value?: string;
        extraKeys?: KeyMap;
        options?: SimpleMDE.Options;
        events?: SimpleMdeToCodemirrorEvents;
        getMdeInstance?: GetMdeInstance;
        getCodemirrorInstance?: GetCodemirrorInstance;
        getLineAndCursor?: GetLineAndCursor;
    }
    export const SimpleMdeReact: React.ForwardRefExoticComponent<SimpleMDEReactProps & React.RefAttributes<HTMLDivElement>>;
    export default SimpleMdeReact;
}
